==> unicore-gateway-6.4.0-2.src.tar.gz: src/main/java/eu/unicore/gateway/ConsignorProducer.java <==
/*
 * Copyright (c) 2007, 2008 ICM Uniwersytet Warszawski All rights reserved.
 * See LICENSE.ICM file for licensing information.
 *
 * Created on May 22, 2007
 * Author: K. Benedyczak <golbi@mat.umk.pl>
 */

package eu.unicore.gateway;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.security.KeyStore;
import java.security.KeyStoreException;
import java.security.NoSuchAlgorithmException;
import java.security.PrivateKey;
import java.security.UnrecoverableKeyException;
import java.security.cert.CertificateException;
import java.security.cert.X509Certificate;
import java.util.ArrayList;
import java.util.Enumeration;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;
import org.apache.log4j.Logger;

import javax.xml.stream.XMLEventFactory;
import javax.xml.stream.XMLEventReader;
import javax.xml.stream.XMLInputFactory;
import javax.xml.stream.XMLStreamException;
import javax.xml.stream.events.XMLEvent;

import org.apache.xmlbeans.XmlObject;
import org.codehaus.stax2.evt.XMLEventFactory2;

import eu.unicore.saml.SAMLConstants.AuthNClasses;
import eu.unicore.security.UnicoreSecurityFactory;
import eu.unicore.security.consignor.ConsignorAPI;
import eu.unicore.security.consignor.ConsignorAssertion;
import eu.unicore.security.dsig.DSigException;
import xmlbeans.org.oasis.saml2.assertion.AssertionDocument;

import com.ctc.wstx.stax.WstxEventFactory;

import eu.unicore.gateway.properties.SecurityProperties;
import eu.unicore.gateway.soap.SoapVersion;
import eu.unicore.gateway.util.LogUtil;

/**
 * Registry and producer of consignor assertions. Assertions are returned as
 * XMLEvents lists to ease insertion to header in main GatewayHandler.
 * If there is no header that this code can also generate wrapping header element.
 * Assertions are cached.
 * @author K. Benedyczak
 */
public class ConsignorProducer implements IConsignorProducer
{
	private static final Logger log =LogUtil.getLogger(LogUtil.GATEWAY,ConsignorProducer.class);
	//last CACHE_SIZE tokens are cached (but only if not-signed mode is on).
	private static final int CACHE_SIZE = 8;
	private String myDN;
	private PrivateKey myKey;
	private int negativeTolerance, validity;
	private boolean doSigned;
	private LinkedHashMap<X509Certificate, List<XMLEvent>> cache;

	public ConsignorProducer(boolean doSigned, int negativeTolerance,
			int validity, File securityProperties) 
		throws KeyStoreException, NoSuchAlgorithmException, CertificateException, 
		FileNotFoundException, IOException
	{
		this.negativeTolerance = negativeTolerance;
		this.validity = validity;
		this.doSigned=doSigned;
		cache = new LinkedHashMap<X509Certificate, List<XMLEvent>>(CACHE_SIZE + 2, 1.0f, true)
		{
			private static final long serialVersionUID = 1L;

			protected boolean removeEldestEntry(
					Map.Entry<X509Certificate, List<XMLEvent>> eldest)
			{
				return size() > CACHE_SIZE;
			}
		};
		init(securityProperties);
	}
	
	private void init(File securityProperties) throws KeyStoreException, NoSuchAlgorithmException, 
		CertificateException, FileNotFoundException, IOException
	{
		SecurityProperties mySecurityConf = new SecurityProperties(securityProperties);
		String ksPath = mySecurityConf.getKeystore();
		String pswd = mySecurityConf.getKeystorePassword();
		KeyStore ks = KeyStore.getInstance(mySecurityConf.getKeystoreType());
		ks.load(new FileInputStream(ksPath), pswd.toCharArray());
		Enumeration<String> aliases = ks.aliases();
		String alias = null;
		while (aliases.hasMoreElements())
		{
			alias = aliases.nextElement();
			if (ks.isKeyEntry(alias))
				break;
		}
		if (alias == null)
			throw new KeyStoreException("No private key in keystore");
		X509Certificate cert = (X509Certificate) ks.getCertificate(alias);
		myDN = cert.getSubjectX500Principal().getName();
		myKey = null;
		if (doSigned)
		{
			try
			{
				myKey = (PrivateKey) 
					ks.getKey(alias, pswd.toCharArray());
			} catch (UnrecoverableKeyException e)
			{
				throw new KeyStoreException("Can't recover key from" +
						" keystore");
			} catch (ClassCastException e)
			{
				throw new KeyStoreException("The key in keysore" +
						" is not a private key");
			}
		}
	}
	
	public List<XMLEvent> getConsignorAssertion(X509Certificate[] certChain, SoapVersion soapVer) throws Exception
	{
		List<XMLEvent>ret=null;
		
		X509Certificate cert = (certChain == null) ? null: certChain[0];
		
		if (myKey == null)
		{
			ret = cacheGet(cert);
			if(ret!=null)return ret;
		}
		
		ret = new ArrayList<XMLEvent>();
		XMLEventFactory2 events=new WstxEventFactory();
		invokeInternal(certChain, events, ret);
		if (myKey == null){
			cacheAdd(cert, ret);
		}
		return ret;
	}
	
	private void invokeInternal(X509Certificate []cert, XMLEventFactory eventFactory, List<XMLEvent> events) 
		throws XMLStreamException, DSigException 
	{
		ConsignorAPI engine = UnicoreSecurityFactory.getConsignorAPI();
		ConsignorAssertion assertion;
		if (cert != null) 
		{
			if (myKey != null)
				assertion = engine.generateConsignorToken(myDN, cert, myKey,
					negativeTolerance, validity,
					AuthNClasses.TLS);
			else
				assertion = engine.generateConsignorToken(myDN, cert, 
						AuthNClasses.TLS);
		} else
		{
			log.debug("Creating ANONYMOUS SAML consignor token");
			if (myKey != null)
				assertion = engine.generateConsignorToken(myDN, 
						negativeTolerance, validity, myKey);
			else
				assertion = engine.generateConsignorToken(myDN);
		}
		AssertionDocument assertionDoc = assertion.getXML();
		save(assertionDoc, eventFactory, events);
		if(log.isDebugEnabled())log.debug("Consignor token: "+assertionDoc);
	}

	
	//write an XmlObject into an XMLEvent list
	private void save(XmlObject obj, XMLEventFactory eventFactory, List<XMLEvent> events)throws XMLStreamException{
		XMLEventReader reader=XMLInputFactory.newInstance().createXMLEventReader(obj.newXMLStreamReader());
		while(reader.hasNext()){
			XMLEvent ev=reader.nextEvent();
			if(ev.isEndDocument())break;
			events.add(ev);
		}
	}
	
	private synchronized void cacheAdd(X509Certificate key, List<XMLEvent> value)
	{
		cache.put(key, value);
	}

	private synchronized List<XMLEvent> cacheGet(X509Certificate key)
	{
		return cache.get(key);
	}
}

==> unicore-gateway6-4.3.0-2.src.tar.gz: src/main/java/eu/unicore/gateway/ConsignorProducer.java <==
/*
 * Copyright (c) 2007, 2008 ICM Uniwersytet Warszawski All rights reserved.
 * See LICENSE.ICM file for licensing information.
 *
 * Created on May 22, 2007
 * Author: K. Benedyczak <golbi@mat.umk.pl>
 */

package eu.unicore.gateway;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.security.KeyStore;
import java.security.KeyStoreException;
import java.security.NoSuchAlgorithmException;
import java.security.PrivateKey;
import java.security.UnrecoverableKeyException;
import java.security.cert.CertificateException;
import java.security.cert.X509Certificate;
import java.util.ArrayList;
import java.util.Enumeration;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;
import org.apache.log4j.Logger;

import javax.xml.stream.XMLEventFactory;
import javax.xml.stream.XMLEventReader;
import javax.xml.stream.XMLInputFactory;
import javax.xml.stream.XMLStreamException;
import javax.xml.stream.events.XMLEvent;

import org.apache.xmlbeans.XmlObject;
import org.codehaus.stax2.evt.XMLEventFactory2;

import eu.unicore.samly2.SAMLConstants.AuthNClasses;
import eu.unicore.security.UnicoreSecurityFactory;
import eu.unicore.security.consignor.ConsignorAPI;
import eu.unicore.security.consignor.ConsignorAssertion;
import eu.unicore.security.dsig.DSigException;
import xmlbeans.org.oasis.saml2.assertion.AssertionDocument;

import com.ctc.wstx.stax.WstxEventFactory;

import eu.unicore.gateway.properties.SecurityProperties;
import eu.unicore.gateway.soap.SoapVersion;
import eu.unicore.gateway.util.LogUtil;

/**
 * Registry and producer of consignor assertions. Assertions are returned as
 * XMLEvents lists to ease insertion to header in main GatewayHandler.
 * If there is no header that this code can also generate wrapping header element.
 * Assertions are cached.
 * @author K. Benedyczak
 */
public class ConsignorProducer implements IConsignorProducer
{
	private static final Logger log =LogUtil.getLogger(LogUtil.GATEWAY,ConsignorProducer.class);
	//last CACHE_SIZE tokens are cached (but only if not-signed mode is on).
	private static final int CACHE_SIZE = 8;
	private String myDN;
	private PrivateKey myKey;
	private int negativeTolerance, validity;
	private boolean doSigned;
	private LinkedHashMap<X509Certificate, List<XMLEvent>> cache;

	public ConsignorProducer(boolean doSigned, int negativeTolerance,
			int validity, File securityProperties) 
		throws KeyStoreException, NoSuchAlgorithmException, CertificateException, 
		FileNotFoundException, IOException
	{
		this.negativeTolerance = negativeTolerance;
		this.validity = validity;
		this.doSigned=doSigned;
		cache = new LinkedHashMap<X509Certificate, List<XMLEvent>>(CACHE_SIZE + 2, 1.0f, true)
		{
			private static final long serialVersionUID = 1L;

			protected boolean removeEldestEntry(
					Map.Entry<X509Certificate, List<XMLEvent>> eldest)
			{
				return size() > CACHE_SIZE;
			}
		};
		init(securityProperties);
	}
	
	private void init(File securityProperties) throws KeyStoreException, NoSuchAlgorithmException, 
		CertificateException, FileNotFoundException, IOException
	{
		SecurityProperties mySecurityConf = new SecurityProperties(securityProperties);
		String ksPath = mySecurityConf.getKeystore();
		String pswd = mySecurityConf.getKeystorePassword();
		KeyStore ks = KeyStore.getInstance(mySecurityConf.getKeystoreType());
		FileInputStream istream=new FileInputStream(ksPath);
		try{
			ks.load(istream, pswd.toCharArray());
		}
		finally{
			istream.close();
		}
		Enumeration<String> aliases = ks.aliases();
		String alias = null;
		while (aliases.hasMoreElements())
		{
			alias = aliases.nextElement();
			if (ks.isKeyEntry(alias))
				break;
		}
		if (alias == null)
			throw new KeyStoreException("No private key in keystore");
		X509Certificate cert = (X509Certificate) ks.getCertificate(alias);
		myDN = cert.getSubjectX500Principal().getName();
		myKey = null;
		if (doSigned)
		{
			try
			{
				myKey = (PrivateKey) 
					ks.getKey(alias, pswd.toCharArray());
			} catch (UnrecoverableKeyException e)
			{
				throw new KeyStoreException("Can't recover key from" +
						" keystore");
			} catch (ClassCastException e)
			{
				throw new KeyStoreException("The key in keysore" +
						" is not a private key");
			}
		}
	}
	
	public List<XMLEvent> getConsignorAssertion(X509Certificate[] certChain, SoapVersion soapVer) throws Exception
	{
		List<XMLEvent>ret=null;
		
		X509Certificate cert = (certChain == null) ? null: certChain[0];
		
		if (myKey == null)
		{
			ret = cacheGet(cert);
			if(ret!=null)return ret;
		}
		
		ret = new ArrayList<XMLEvent>();
		XMLEventFactory2 events=new WstxEventFactory();
		invokeInternal(certChain, events, ret);
		if (myKey == null){
			cacheAdd(cert, ret);
		}
		return ret;
	}
	
	private void invokeInternal(X509Certificate []cert, XMLEventFactory eventFactory, List<XMLEvent> events) 
		throws XMLStreamException, DSigException 
	{
		ConsignorAPI engine = UnicoreSecurityFactory.getConsignorAPI();
		ConsignorAssertion assertion;
		if (cert != null) 
		{
			if (myKey != null)
				assertion = engine.generateConsignorToken(myDN, cert, myKey,
					negativeTolerance, validity,
					AuthNClasses.TLS);
			else
				assertion = engine.generateConsignorToken(myDN, cert, 
						AuthNClasses.TLS);
		} else
		{
			log.debug("Creating ANONYMOUS SAML consignor token");
			if (myKey != null)
				assertion = engine.generateConsignorToken(myDN, 
						negativeTolerance, validity, myKey);
			else
				assertion = engine.generateConsignorToken(myDN);
		}
		AssertionDocument assertionDoc = assertion.getXML();
		save(assertionDoc, eventFactory, events);
		if(log.isDebugEnabled())log.debug("Consignor token: "+assertionDoc);
	}

	
	//write an XmlObject into an XMLEvent list
	private void save(XmlObject obj, XMLEventFactory eventFactory, List<XMLEvent> events)throws XMLStreamException{
		XMLEventReader reader=XMLInputFactory.newInstance().createXMLEventReader(obj.newXMLStreamReader());
		while(reader.hasNext()){
			XMLEvent ev=reader.nextEvent();
			if(ev.isEndDocument())break;
			events.add(ev);
		}
	}
	
	private synchronized void cacheAdd(X509Certificate key, List<XMLEvent> value)
	{
		cache.put(key, value);
	}

	private synchronized List<XMLEvent> cacheGet(X509Certificate key)
	{
		return cache.get(key);
	}
}
